#
# spec file for package ghc-data-accessor
#
# Copyright (c) 2016 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via http://bugs.opensuse.org/
#


%global pkg_name data-accessor
Name:           ghc-%{pkg_name}
Version:        0.2.2.7
Release:        1.11
Summary:        Utilities for accessing and manipulating fields of records
License:        BSD-3-Clause
Group:          System/Libraries
Url:            https://hackage.haskell.org/package/%{pkg_name}
Source0:        https://hackage.haskell.org/package/%{pkg_name}-%{version}/%{pkg_name}-%{version}.tar.gz
BuildRequires:  ghc-Cabal-devel
# Begin cabal-rpm deps:
BuildRequires:  ghc-array-devel
BuildRequires:  ghc-containers-devel
BuildRequires:  ghc-rpm-macros
BuildRequires:  ghc-transformers-devel
BuildRoot:      %{_tmppath}/%{name}-%{version}-build
# End cabal-rpm deps

%description
In Haskell 98 the name of a record field is automatically also the name of a
function which gets the value of the according field. E.g. if we have

data Pair a b = Pair first :: a, second :: b

then

> first :: Pair a b -> a > second :: Pair a b -> b

However for setting or modifying a field value we need to use some syntactic
sugar, which is often clumsy.

modifyFirst :: (a -> a) -> (Pair a b -> Pair a b) modifyFirst f r@(Pair first=a
) = r first = f a

With this package you can define record field accessors which allow setting,
getting and modifying values easily. The package clearly demonstrates the power
of the functional approach: You can combine accessors of a record and
sub-records, to make the access look like the fields of the sub-record belong
to the main record.

Example:

> *Data.Accessor.Example> (first^:second^=10) (('b',7),"hallo") >
(('b',10),"hallo")

You can easily manipulate record fields in a 'Control.Monad.State.State' monad,
you can easily code 'Show' instances that use the Accessor syntax and you can
parse binary streams into records. See 'Data.Accessor.Example' for
demonstration of all features.

It would be great if in revised Haskell versions the names of record fields are
automatically 'Data.Accessor.Accessor's rather than plain 'get' functions.
For now, the package 'data-accessor-template' provides Template Haskell
functions for automated generation of 'Data.Acesssor.Accessor's. See also the
other 'data-accessor' packages that provide an Accessor interface to other data
types. The package 'enumset' provides accessors to bit-packed records.

For similar packages see 'lenses' and 'fclabel'. A related concept are editors
<http://conal.net/blog/posts/semantic-editor-combinators/>. Editors only
consist of a modify method (and 'modify' applied to a 'const' function is a
'set' function). This way, they can modify all function values of a function at
once, whereas an accessor can only change a single function value, say, it can
change 'f 0 = 1' to 'f 0 = 2'. This way, editors can even change the type of a
record or a function. An Arrow instance can be defined for editors, but for
accessors only a Category instance is possible ('(.)' method). The reason is
the 'arr' method of the 'Arrow' class, that conflicts with the two-way nature
(set and get) of accessors.

%package devel
Summary:        Haskell %{pkg_name} library development files
Group:          Development/Libraries/Other
Requires:       %{name} = %{version}-%{release}
Requires:       ghc-compiler = %{ghc_version}
Requires(post): ghc-compiler = %{ghc_version}
Requires(postun): ghc-compiler = %{ghc_version}

%description devel
This package provides the Haskell %{pkg_name} library development files.

%prep
%setup -q -n %{pkg_name}-%{version}


%build
%ghc_lib_build


%install
%ghc_lib_install


%post devel
%ghc_pkg_recache

%postun devel
%ghc_pkg_recache

%files -f %{name}.files
%defattr(-,root,root,-)
%doc LICENSE

%files devel -f %{name}-devel.files
%defattr(-,root,root,-)

%changelog
* Sun Jul 10 2016 psimons@suse.com
- Update to version 0.2.2.7 revision 0 with cabal2obs.
